/*
Copyright Caroline et Serge COUDÉ, (04/2012) 

serge.coude@gmail.com caroline.coude@laposte.net

Ce logiciel est un programme informatique faisant partie d'une suite de 
logiciels 'TbiSuite' à destination des enseignants et des élèves usagés des 
ordinateurs et Tableau Numérique Interactif / Tableau Blanc Interactif. 

Ce logiciel est régi par la licence CeCILL v2 soumise au droit français et
respectant les principes de diffusion des logiciels libres. Vous pouvez
utiliser, modifier et/ou redistribuer ce programme sous les conditions
de la licence CeCILL v2 telle que diffusée par le CEA, le CNRS et l'INRIA 
sur le site "http://www.cecill.info".

En contrepartie de l'accessibilité au code source et des droits de copie,
de modification et de redistribution accordés par cette licence, il n'est
offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
seule une responsabilité restreinte pèse sur l'auteur du programme,  le
titulaire des droits patrimoniaux et les concédants successifs.

A cet égard  l'attention de l'utilisateur est attirée sur les risques
associés au chargement,  à l'utilisation,  à la modification et/ou au
développement et à la reproduction du logiciel par l'utilisateur étant 
donné sa spécificité de logiciel libre, qui peut le rendre complexe à 
manipuler et qui le réserve donc à des développeurs et des professionnels
avertis possédant  des  connaissances  informatiques approfondies.  Les
utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
logiciel à leurs besoins dans des conditions permettant d'assurer la
sécurité de leurs systèmes et ou de leurs données et, plus généralement, 
à l'utiliser et l'exploiter dans les mêmes conditions de sécurité. 

Le fait que vous puissiez accéder à cet en-tête signifie que vous avez 
pris connaissance de la licence CeCILL v2, et que vous en avez accepté les
termes.

*/

package net.sergecoude.TbiGeneral;

import java.util.*;
import java.io.*;
import net.n3.nanoxml.*;
import java.awt.event.*;
import javax.swing.*;

public class Evaluation { 

  private final static long serialVersionUID = 1L;
  
  public static int DATE_ASC = 0;
  public static int DATE_DESC = 1; 
  private static ArrayList<ParametreDeTri> _lesParametresDeTri;
  
  protected TbiApp _app;
  protected Calendar _dateStart;
  protected Calendar _dateStop;
  protected Classe _classe;
  protected Vector<Epreuve> _lesEpreuves;
  protected boolean _estEnregistree = false;
  protected String _nom;
  protected String _cle;
  protected String _chemin;
  protected String _versionApplication;
  protected String _langue = "Francais";

  public Evaluation(TbiApp app) {
    _app = app;
    _classe = _app.getClasse();
    _lesEpreuves = new Vector<Epreuve>();
    _nom = "";
    _versionApplication = "";
  }
  
  public Evaluation(TbiApp app, String chemin) throws Exception {
    this(app);
    _chemin = chemin;
    _classe = null;
    BufferedReader br = new BufferedReader(new InputStreamReader(new FileInputStream(chemin)));
    String ctn64 = "";
		String ligne;
		while ((ligne = br.readLine()) != null)
				ctn64 += ligne;
		br.close(); 
    String ctnXML = Base64.decodeString(ctn64);
   	IXMLParser parser = XMLParserFactory.createDefaultXMLParser();
	  IXMLReader reader = new StdXMLReader(new StringReader(ctnXML));
		parser.setReader(reader);
		IXMLElement xml = (IXMLElement) parser.parse(); // <evaluation>
		chargeEvaluation(xml);
	}
  
  public void chargeEvaluation(IXMLElement xml) throws Exception {
		IXMLElement xNom = xml.getFirstChildNamed("nom");
		if (xNom != null) {
		  setNom(xNom.getContent());
    } else 
      throwNewException("Le nom de l'évaluation n'est pas indiqué... [" + _chemin + "]");
		IXMLElement xCle = xml.getFirstChildNamed("cle");
		if (xCle != null) {
		  setCle(xCle.getContent());
    } else 
      throwNewException("La clé de l'évaluation n'est pas indiquée... [" + _chemin + "]");
		IXMLElement xDebut = xml.getFirstChildNamed("datedebut");
		if (xDebut != null) {
		  Calendar date = Calendar.getInstance();
		  date.setTimeInMillis(Long.parseLong(xDebut.getContent()));
		  setDateStart(date);
    } else 
      throwNewException("La date de début d'évaluation n'est pas indiquée... [" + _chemin + "]");
  	IXMLElement xFin = xml.getFirstChildNamed("datefin");
		if (xFin != null) {
		  Calendar date = Calendar.getInstance();
		  date.setTimeInMillis(Long.parseLong(xFin.getContent()));
		  setDateStop(date);
    } else
      throwNewException("La date de fin d'évaluation n'est pas indiquée... [" + _chemin + "]");
    IXMLElement xClasse = xml.getFirstChildNamed("classe");
		if (xClasse != null) {
		  _classe = _app.getClasseById(xClasse.getContent());
		  if (_classe == null) 
		    throwNewException("La classe de l'évaluation n'est pas présente... [" + _chemin + "]");
    } else
      throwNewException("La classe de l'évaluation n'est pas indiquée... [" + _chemin + "]");
    IXMLElement xVersion = xml.getFirstChildNamed("version");
		if (xVersion != null) {
		  setVersionApplication(xVersion.getContent());
    }
		IXMLElement xLangue = xml.getFirstChildNamed("langue");
		if (xLangue != null) {
		  setLangue(xLangue.getContent());
    }     
    IXMLElement xEpreuves = xml.getFirstChildNamed("epreuves");
		if (xEpreuves != null) {
      Vector le = xEpreuves.getChildrenNamed("epreuve");
      if (le != null) {
        for(int i = 0; i < le.size(); i++) {
          IXMLElement e = (IXMLElement)(le.elementAt(i));
          try {
            addEpreuve(getNewEpreuve(_classe, e));
          } catch(Exception exception) {
            throwNewException(exception.toString());
          }
  			}
  		} else 
        throwNewException("Aucune épreuve trouvée dans l'évaluation... [" + _chemin + "]");  			  
    }
    _estEnregistree = true;
  }
  
  public void throwNewException(String e) throws Exception {
    throw new Exception("Evaluation " + _chemin + " : " + e);
  }
  
  public Epreuve getNewEpreuve(Classe classe, IXMLElement xmlEpreuve) throws Exception {
    return _app.getNewEpreuve(classe, xmlEpreuve);
  }
  
  public Classe getClasse() {
    return _classe;
  }
  
  public Epreuve addEpreuve(Epreuve ep) {
    _lesEpreuves.add(ep);
    return ep;
  }
  
  public Vector<Epreuve> getLesEpreuves() {
    return _lesEpreuves;
  }
  
  public Epreuve getEpreuve() {
    if (_lesEpreuves.size() > 0)
      return _lesEpreuves.lastElement();
    return null;
  }
  
  public void setEnregistree(boolean oui) {
    _estEnregistree = oui;
  }
  
  public void setNom(String nom) {
    _nom = nom;
  }
  
  public String getNom() {
    return _nom;
  }

  public void setVersionApplication(String version) {
    _versionApplication = version;
  }
  
  public String getVersionApplication() {
    return _versionApplication;
  }
  
  public int getNbExercices() {
    if (_lesEpreuves != null && _lesEpreuves.size() > 0) {
      Epreuve ep = _lesEpreuves.elementAt(0);
      if (ep != null) {
        Vector<Exercice> lesExercices = ep.getLesExercices();
        if (lesExercices != null)
          return lesExercices.size();
      }
    }
    return 0;
  }
  
  public String getChemin() {
    return _chemin;
  }
  
  public void setCle(String cle) {
    _cle = cle;
  }
  
  public String getCle() {
    return _cle;
  }

  public void setLangue(String langue) {
    _langue = langue;
  }
  
  public String getLangue() {
    return _langue;
  }
  
  public String getCodeNiveauDifficulte() {
    return "";
  }
  
  public boolean estEnregistree() {
    return _estEnregistree;
  }

  public void start() {
   setDateStart(Calendar.getInstance());
  }
  
  public void setDateStart(Calendar date) {
    _dateStart = date;
  }
  
  public void stop() {
    setDateStop(Calendar.getInstance());
  }
  
  public void setDateStop(Calendar date) {
    _dateStop = date;
  }
  
  public boolean peutSeTerminer() {
    if (_dateStop != null)
      return true;
    if (getEpreuve() != null)
      return getEpreuve().estStopee();
    return true;
  }
  
  public String toXmlDebut() {
    String xml = "<evaluation>";
    xml += "<nom>" + _nom + "</nom>";
    xml += "<cle>" + _cle + "</cle>";
    xml += "<datedebut>" + _dateStart.getTimeInMillis() + "</datedebut>";
    xml += "<datefin>" + _dateStop.getTimeInMillis() + "</datefin>";
    xml += "<classe>" + _classe.getId() + "</classe>";
    xml += "<version>" + _versionApplication + "</version>";
    xml += "<nbexercices>" + getNbExercices() + "</nbexercices>";
    xml += "<langue>" + getLangue() + "</langue>";
    return xml;
  }
  
  public String toXmlEpreuves() {
    String xml = "<epreuves>";
    for(int e = 0; e < _lesEpreuves.size(); e++)
      xml += _lesEpreuves.elementAt(e).toXml();
    xml += "</epreuves>";
    return xml;
  }

  public String toXmlFin() {
    return "</evaluation>";
  }
  
  public String toXml() {
    return toXmlDebut() + toXmlEpreuves() + toXmlFin();
  }
  
  public void annuleEpreuve(Epreuve ep) {
    ep.stop();  
    _lesEpreuves.removeElement(ep);
  }
  
  public boolean contientDesEnregistrements() {
    for(int i =  0; i < _lesEpreuves.size(); i++) {
      Vector<Exercice> lesExercices = _lesEpreuves.elementAt(i).getLesExercices();
      for(int j =  0; j < lesExercices.size(); j++) {
        Vector<Enregistrement> lesEnregistrements = lesExercices.elementAt(j).getLesEnregistrements();
        if (lesEnregistrements.size() > 0)
          return true;
      }        
    }
    return false;  
  }
  
  public void enregistrer(String cheminRepEval) throws Exception {
    if (!contientDesEnregistrements())
      return;
    String ctn = "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"yes\" ?>\n";
    String nomFichier = genereNomFichier();
    ctn += toXml();
    ctn = Base64.encodeString(ctn);
    File repEval = new File(cheminRepEval);
    if (!repEval.exists()) 
      repEval.mkdirs();
    File fichier = new File(repEval, nomFichier);
    PrintWriter out = new PrintWriter(new BufferedWriter(new OutputStreamWriter(new FileOutputStream(fichier, false),"UTF-8")));
    out.println(ctn);
    out.close();
    _estEnregistree = true; 
  }
  
  public String genereNomFichier() {
    String aaaa = "" + _dateStart.get(Calendar.YEAR);
    String mm = "" + (_dateStart.get(Calendar.MONTH) < 9 ? "0" : "") + (_dateStart.get(Calendar.MONTH)+1);
    String jj = "" + (_dateStart.get(Calendar.DAY_OF_MONTH) < 10 ? "0" : "")  + _dateStart.get(Calendar.DAY_OF_MONTH);
    String hh = "" + (_dateStart.get(Calendar.HOUR_OF_DAY) < 10 ? "0" : "") + _dateStart.get(Calendar.HOUR_OF_DAY);
    String mn = "" + (_dateStart.get(Calendar.MINUTE) < 10 ? "0" : "") + _dateStart.get(Calendar.MINUTE);
    String ss = "" + (_dateStart.get(Calendar.SECOND) < 10 ? "0" : "") + _dateStart.get(Calendar.SECOND);   
    return aaaa+mm+jj+"-"+hh+mn+ss+"_"+_nom+"-eval.xml";   
  }
  
  protected void genereNom(TbiApp app) {
    _nom = "Evaluation";
  }
  
  public Calendar getDateStart() {
    return _dateStart;
  }

  public Calendar getDateStop() {
    return _dateStop;
  }
  
  public static ArrayList<ParametreDeTri> getParametresDeTri() {
    if (_lesParametresDeTri == null) {
      _lesParametresDeTri = new ArrayList<ParametreDeTri>();
      _lesParametresDeTri.add(new ParametreDeTri(DATE_ASC, "Date ancienne->récente"));
      _lesParametresDeTri.add(new ParametreDeTri(DATE_DESC, "Date récente->ancienne"));
    }
    return _lesParametresDeTri;
  } 

  public static Comparator<Evaluation> getComparator(ArrayList<ParametreDeTri> paramTri) {
    return new EvaluationComparator(paramTri);
  }
  
  private static class EvaluationComparator implements Comparator<Evaluation> {
    private ArrayList<ParametreDeTri> _parametres;

    private EvaluationComparator(ArrayList<ParametreDeTri> parametres) {
      _parametres = parametres;
    }

    public int compare(Evaluation e1, Evaluation e2) {
      int comparaison = 0;
      for(int i = 0; i < _parametres.size(); i++ ) {
        int idparam = _parametres.get(i).getId();
        if (idparam == Evaluation.DATE_ASC)
          comparaison = (e1.getDateStart().getTimeInMillis() < e2.getDateStart().getTimeInMillis() ? -1 : (e1.getDateStart().getTimeInMillis() == e2.getDateStart().getTimeInMillis() ? 0 : 1));
        if (idparam == Evaluation.DATE_DESC)
          comparaison = (e1.getDateStart().getTimeInMillis() < e2.getDateStart().getTimeInMillis() ? 1 : (e1.getDateStart().getTimeInMillis() == e2.getDateStart().getTimeInMillis() ? 0 : -1));
        if (comparaison != 0) return comparaison;
      }
      return 0;
    }
  }
  
  public boolean elevePresent(Eleve eleve) {
    for(int e = 0; e < _lesEpreuves.size(); e++ ) 
      if (_lesEpreuves.elementAt(e).getEleve().getId().equals(eleve.getId()))
        return true;
    return false;
  }
  
  public int getDureeReponse(boolean max) {
    int ret = -1;
    for(int i =  0; i < _lesEpreuves.size(); i++) {
      Vector<Exercice> lesExercices = _lesEpreuves.elementAt(i).getLesExercices();
      for(int j =  0; j < lesExercices.size(); j++) {
        Vector<Enregistrement> lesEnregistrements = lesExercices.elementAt(j).getLesEnregistrements();
        for(int k =  0; k < lesEnregistrements.size(); k++) {
          Enregistrement enr = lesEnregistrements.elementAt(k);
//          System.out.println( _lesEpreuves.elementAt(i).getEleve().getNomUsuel() + " => " + enr.getDuree());
          if (enr.estUnSucces()) {
            if (ret == -1 || 
                (max && ret < enr.getDuree()) ||
                (!max && ret > enr.getDuree())) {
              ret = enr.getDuree();
            }
          }
        }
      }        
    }
    return ret;  
  }
    
}
