/*
Copyright Caroline et Serge COUDÉ, (09/2012) 

serge.coude@gmail.com caroline.coude@laposte.net

Ce logiciel est un programme informatique faisant partie d'une suite de 
logiciels 'TbiSuite' à destination des enseignants et des élèves usagés des 
ordinateurs et Tableau Numérique Interactif / Tableau Blanc Interactif. 

Ce logiciel est régi par la licence CeCILL v2 soumise au droit français et
respectant les principes de diffusion des logiciels libres. Vous pouvez
utiliser, modifier et/ou redistribuer ce programme sous les conditions
de la licence CeCILL v2 telle que diffusée par le CEA, le CNRS et l'INRIA 
sur le site "http://www.cecill.info".

En contrepartie de l'accessibilité au code source et des droits de copie,
de modification et de redistribution accordés par cette licence, il n'est
offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
seule une responsabilité restreinte pèse sur l'auteur du programme,  le
titulaire des droits patrimoniaux et les concédants successifs.

A cet égard  l'attention de l'utilisateur est attirée sur les risques
associés au chargement,  à l'utilisation,  à la modification et/ou au
développement et à la reproduction du logiciel par l'utilisateur étant 
donné sa spécificité de logiciel libre, qui peut le rendre complexe à 
manipuler et qui le réserve donc à des développeurs et des professionnels
avertis possédant  des  connaissances  informatiques approfondies.  Les
utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
logiciel à leurs besoins dans des conditions permettant d'assurer la
sécurité de leurs systèmes et ou de leurs données et, plus généralement, 
à l'utiliser et l'exploiter dans les mêmes conditions de sécurité. 

Le fait que vous puissiez accéder à cet en-tête signifie que vous avez 
pris connaissance de la licence CeCILL v2, et que vous en avez accepté les
termes.

*/

package net.sergecoude.TbiGeneral;

import javax.swing.*;
import java.awt.*;
import java.net.*;
import java.io.*;
import java.awt.event.*;
import javax.swing.border.*;
import net.n3.nanoxml.*; 
import java.util.*;

public class TbiApp extends JFrame implements ActionListener, ComponentListener {

  private final static long serialVersionUID = 1L;
  
  protected static TbiApp _this = null;
	protected TbiConfGenerale _configuration;
	protected HashMap<Integer, Niveau> _lesNiveaux;
	private String _fichierConfNiveaux = "niveaux.xml";
	public static int MESSAGE_ERREUR = 1;
	public static int MESSAGE_INFORMATION = 2;
	public static int MESSAGE_AVERTISSEMENT = 3;
  protected Message _message;
  protected GraphicsDevice _ecran = null;
  protected boolean _ecranPrincipal = true;
  protected boolean _init = false;
  protected Color _cFond = Color.white;
  protected JPanel _pFrame;
  protected Box _pHaut;
  protected JLayeredPane _pLayered;
  protected PanelMain _pMain;
  protected JButton _bQuitter;
  protected JButton _bAPropos;
  protected JButton _bIconifier;
  private bouton _logo;
  protected DefaultListModel _lesClasses = null;
  protected OggPlayer _oggPlayer = new OggPlayer(this);
  protected java.util.Timer _timer; 
  protected PanelSelectEcran _pSelectEcran;
  protected PanelSelectClasseGen _pSelectClasseGen;
  protected PanelSelectEleveGen _pSelectEleveGen;
  protected PanelConfig _pConfig; // A instancier dans TbiApp dérivée
  protected PanelJeu _pTableauJeu; // A instancier dans TbiApp dérivée 
  protected PanelTitre _pTitre; // A instancier dans TbiApp dérivée
  protected String _fichierConfGenApp = "TbiGeneralApp.conf";
  protected String _fichierConfGen = "TbiGeneral.conf"; 
  protected String _fichierConf = "";  // à définir obligatoirement pour chaque application
  protected Classe _classe;
  protected Eleve _eleve;
  protected Log _log = null;
  protected static ApplicationInfo _applicationInfo = new ApplicationInfo();
  protected Vector<String> _panneauxApresSelectClasse = new Vector<String>();
  protected Vector<String> _panneauxApresSelectEleve = new Vector<String>();;
  protected Epreuve _epreuve = null;
  protected Vector<Evaluation> _lesEvaluations = null;
  protected Evaluation _evaluationCourante = null;
  protected Skin _skin = null;
  protected HashMap<String,Skin> _lesSkins = null;

  static protected int _largeurPhoto = 200;
  static protected int _hauteurPhoto = 200;
  static protected Dimension _dimensionPhoto = new Dimension(_largeurPhoto, _hauteurPhoto);
  static protected int _largeurLogo = 75;
  static protected int _hauteurLogo = 75;
  static protected Dimension _dimensionLogo = new Dimension(_largeurLogo, _hauteurLogo);
  static protected int _margeAffichage = 10;
  
  // Définition des variables globales
  static public String _V_UtilisationTBI = "tbi";
  protected boolean _avecTbi = false;
  static public String _V_TailleSouris = "taillecurseur";
  protected int _tailleCurseur = 3;
  static public String _V_LargeurTBI = "largeur-ecran-tbi";
  protected int _largeurTbi = 1024;
  static public String _V_HauteurTBI = "hauteur-ecran-tbi";
  protected int _hauteurTbi = 768;
  static public String _V_CoordXTBI = "coordx-ecran-tbi";
  protected int _xTbi = 0;
  static public String _V_CoordYTBI = "coordy-ecran-tbi";
  protected int _yTbi = 0;
  static public String _V_UtilisationDesClasses = "utilisationdesclasses";
  protected boolean _utilisationDesClasses = false;  
  static public String _V_CheminDesClasses = "chemindesclasses";
  protected String _cheminDesClasses = "";
  static public String _V_NbExercices = "nbexercices";
  protected int _nbExercices = 10;
  static public String _V_NomDuSkin = "nomduskin";
  protected String _nomDuSkin = ".";
  
  public TbiApp(String fichierLog) {
    _this = this;
    System.setProperty( "java.protocol.handler.pkgs", "net.sergecoude.protocol" );
    _log = new Log(this, "logs/" + fichierLog);
		_configuration = new TbiConfGenerale();
		if (!_configuration.chargeConfiguration("0", _fichierConfGenApp, false, false))
      System.exit(-1);
	  if (!_configuration.chargeConfiguration("1", _fichierConfGen, false, false))
      System.exit(-1); 
    _lesSkins =  new HashMap<String,Skin>();
    chargeLesNiveaux();
		_bQuitter = new JButton("Q");
		_bAPropos = new JButton("?");
		_bIconifier = new JButton("_");
		_pLayered = new JLayeredPane();
		_pLayered.setOpaque(false);
		_pFrame = new JPanel();
		_pFrame.setLayout(new BorderLayout());
		_pFrame.setOpaque(false);
		setLayout(null);
		add(_pLayered);
    _pFrame.setBorder(new EmptyBorder(_margeAffichage,_margeAffichage,_margeAffichage,_margeAffichage));     

		_pHaut = Box.createHorizontalBox();
		_message = new Message(this);
		_pHaut.add(_message);
		_pHaut.add(Box.createHorizontalGlue());
		_pHaut.add(_bAPropos);
		_pHaut.add(Box.createHorizontalStrut(15));
		_pHaut.add(_bIconifier);
		_pHaut.add(Box.createHorizontalStrut(15));
		_pHaut.add(_bQuitter);
		_pHaut.setBackground(_cFond);
		_pFrame.add(_pHaut, BorderLayout.NORTH);
		_bQuitter.addActionListener(this);
		_bAPropos.addActionListener(this);
		_bIconifier.addActionListener(this);
		
		_pMain = new PanelMain(this);
		_pFrame.add(_pMain, BorderLayout.CENTER);
		
		try {
      _logo = new bouton(getRessource("/logo-caroline-et-serge-coude.png"));
    } catch (Exception e) {
      log(e);
    }
    _pLayered.add(_pFrame, new Integer(0), 0);
    _pLayered.add(_logo, new Integer(1), 0);
    
 		_pSelectEcran = new PanelSelectEcran(this);
 		_pSelectClasseGen = new PanelSelectClasseGen(this);
 		_pSelectEleveGen = new PanelSelectEleveGen(this);
		_pMain.add(_pSelectEcran,"SelectEcran");
		_pMain.add(_pSelectClasseGen,"SelectClasseGen");
		_pMain.add(_pSelectEleveGen,"SelectEleveGen");
		
		_lesEvaluations = new Vector<Evaluation>();
    setUndecorated(true);
    _init = true;
    addComponentListener(this);
		setFullKiosque();
  }
  
  public void componentHidden(ComponentEvent e) {
    quitter();
  }

  public void componentMoved(ComponentEvent e) {
  }
  
  public void componentResized(ComponentEvent e) {
  }

  public void componentShown(ComponentEvent e) {
  		setIcone(); // à appeler une fois l'instanciation de l'application effectuée
  }  
  
  public void setFullKiosque() {
    if (_init) {
        Toolkit tk = Toolkit.getDefaultToolkit();
  	  	Dimension d = tk.getScreenSize();
        _pFrame.setBounds(0, 0, d.width, d.height);
    		_pLayered.setBounds(0, 0, d.width, d.height);
  	 	  setSize(d.width,d.height);		
  		  setLocation(0,0);
        _logo.setBounds(_margeAffichage,d.height-_margeAffichage-_hauteurLogo,_largeurLogo,_hauteurLogo);
    }
  }
  	
/*	public void setFullKiosque() {
    if (_init) {
      GraphicsEnvironment environment = GraphicsEnvironment.getLocalGraphicsEnvironment();
      GraphicsDevice device = environment.getDefaultScreenDevice();
      if (!device.isFullScreenSupported()) {
        Toolkit tk = Toolkit.getDefaultToolkit();
  	  	Dimension d = tk.getScreenSize();
  	 	  setSize(d.width,d.height);		
  		  setLocation(0,0);
      } else {
        device.setFullScreenWindow(this);
      }
      setVisible(true);
    }
  }  */
  
  protected void setIcone() {
  try {
      ImageIcon imgIcon = new ImageIcon(getRessource("/" + _applicationInfo.getNom() + ".png"));
      Image icone = imgIcon.getImage();  
      setIconImage(icone);
    } catch(Exception e) {
      log(e);
    }
  }
  
  public void majAffichage() { 
    Rectangle bounds = getBoundsEcran();
 	  setSize(bounds.width,bounds.height);		
	  setLocation(bounds.x,bounds.y);
    _pFrame.setBounds(getXTbi(), getYTbi(), getLargeurTbi(), getHauteurTbi());
    _pHaut.setPreferredSize(new Dimension(getLargeurTbi(), getHauteurTbi()/15));
    float ratioLogo = (float)getHauteurTbi() / 768f;
    int hLogo = (int)(_hauteurLogo * ratioLogo);
    int lLogo = (int)(_largeurLogo * ratioLogo);
    _logo.setBounds(getXTbi()+_margeAffichage,getYTbi()+getHauteurTbi()-_margeAffichage-hLogo,lLogo,hLogo);
    _pHaut.revalidate();
    _pFrame.revalidate();
  }
  
  public Rectangle getBoundsEcran() {
    if (_ecran == null) {
      Dimension tailleEcran = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
      return new Rectangle(0,0,tailleEcran.width,tailleEcran.height);
    }
    GraphicsConfiguration gc = _ecran.getDefaultConfiguration();
    return gc.getBounds();
  } 
  // Get size of each screen
  //for (int i=0; i<gs.length; i++) {
  //  DisplayMode dm = gs[i].getDisplayMode();
  //  int screenWidth = dm.getWidth();
  //  int screenHeight = dm.getHeight();
  //}

  public Rectangle getBoundsAffichage() {
    return _pFrame.getBounds();
  }
  
  public Color getCouleurFond() {
    return _cFond;
  }
  
  public Dimension getMainPanelSize() {
    return _pMain.getSize();
  }
  
  public String getRepertoireApplication() {
    return "";
  } 
  
  public void actionPerformed(ActionEvent evt) {
		Object source = evt.getSource();
		if (source == _bQuitter) {
		  quitter();
		}
		if (source == _bAPropos) {
		  aPropos();
		}
		if (source == _bIconifier) {
		  iconifier();
    }
		if (source instanceof ItemEleveGen)
		  selectionneEleve(((ItemEleveGen)source).getEleve(), true);
  }
  
  protected void quitter() {
    if (getUtilisationDesClasses()) {
      if (_evaluationCourante != null) {
        if (!_evaluationCourante.peutSeTerminer()) {
          enregistrerLesEvaluations(); // enregistre les évaluations terminées, pas celle en cours
          String mess = "Une évaluation est en cours...\nÊtes-vous sûr de vouloir quitter " + _applicationInfo.getNom() + " sans l'avoir sauvegardée ?";
          int rep = JOptionPane.showConfirmDialog(_pFrame, mess, "Quitter", JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE);
          if (rep == JOptionPane.NO_OPTION) 
            return;
        } else {
          _evaluationCourante.stop();
          if (!enregistrerLesEvaluations()) {
            int rep = JOptionPane.showConfirmDialog(_pFrame, "Une erreur est survenue lors de l'enregistrement d'une évaluation...\nÊtes-vous sûr de vouloir quitter " + _applicationInfo.getNom() + " ?", "Quitter", JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE);
            if (rep == JOptionPane.NO_OPTION) 
              return;
          } else {
             messageStatique("L'évaluation a été correctement enregistrée. Vous pouvez quitter " + _applicationInfo.getNom() + "...", MESSAGE_INFORMATION);
          }
        }
      }
    }
  	System.exit(0);
  }
  
  protected void aPropos() {
    if (getUtilisationDesClasses()) {
      if (_evaluationCourante != null) {
        message("Une évaluation est en cours...", MESSAGE_AVERTISSEMENT);
        return;
      }
    }  
    setPanneau("Titre");
  }
  
  protected void iconifier() {
    setExtendedState(JFrame.ICONIFIED);
  }  
  
  public void log(String msg) {
    if (_log != null) {
      _log.log(msg);
    }
  }

  public void log(Exception e) {
    if (_log != null) {
      _log.log(e);
    }
  }
  
  public static ApplicationInfo getApplicationInfo() {
    return _applicationInfo;
  }
  
	public void stopTimer() {
	 if (_timer != null) 
	   _timer.cancel();
	 _timer = null;
  }
  
  protected void setSkin(String nom) {
    _skin = _lesSkins.get(nom);
  }
  
  public Skin getSkin() {
    return _skin;
  }
  
  public String getNomDuSkin() {
    if (_skin != null)
      return _skin.getNom();    
    return _nomDuSkin;
  }
  
  public void chargeLesSkins() {
    try {
      _lesSkins.clear();
      File[] fichiers = null;
      File rep = new File(getRepertoireApplication());
      fichiers = rep.listFiles();
      if (fichiers != null) 
        for(int f = 0; f < fichiers.length; f++) 
          if (fichiers[f].isFile() && fichiers[f].getName().startsWith("skin") && fichiers[f].getName().toLowerCase().endsWith(".jar")) {
            String nom = fichiers[f].getName().substring(0,fichiers[f].getName().length()-4);  
            _lesSkins.put(nom, new Skin(this, fichiers[f].getAbsolutePath(), nom));
          }
    } catch(Exception e) {
      messageStatique(e.toString(), MESSAGE_ERREUR);
      System.exit(-1);
    }
  }
  
  public HashMap<String,Skin> getLesSkins() {
    return _lesSkins;
  }
  
  private boolean chargeLesNiveaux() {
		File fichier = new File(_fichierConfNiveaux);
		URL urlConfNiveaux = null;
    if (!fichier.exists()) {
      urlConfNiveaux = getClass().getResource("/" + _fichierConfNiveaux);
      if (urlConfNiveaux == null) {
        message("\nLe fichier des niveaux " + fichier.getAbsolutePath() + " est introuvable ou non accessible...\n",MESSAGE_ERREUR);
        return false;
      }
    }
    try {
      if (urlConfNiveaux == null)
        urlConfNiveaux = fichier.toURI().toURL();
      if (urlConfNiveaux == null) {
        message("\nLe fichier des niveaux " + urlConfNiveaux + " est introuvable ou non accessible...\n",MESSAGE_ERREUR);
        return false;
      }     
      if (_lesNiveaux == null)
        _lesNiveaux = new HashMap<Integer,Niveau>();
      IXMLParser parser = XMLParserFactory.createDefaultXMLParser();
      IXMLReader reader = new StdXMLReader(urlConfNiveaux.openStream());
  	  parser.setReader(reader);
	    IXMLElement xml = (IXMLElement) parser.parse(); // <niveaux>
  	  Vector lesNiveaux = xml.getChildrenNamed("niveau");
	    for(int n = 0; n < lesNiveaux.size(); n++) {
		    XMLElement niv = (XMLElement)(lesNiveaux.elementAt(n));
		      String nom = niv.getAttribute("nom","niveau non défini");
		      int idx = niv.getAttribute("id",0);
		      String style = niv.getAttribute("style","");
		      Niveau niveau = new Niveau(nom, idx, style);
		      _lesNiveaux.put(new Integer(idx), niveau);
		  }
    } catch (MalformedURLException mfue) {
        message("\nLe chemin du fichier des niveaux " + fichier.getAbsolutePath() + " n'est pas valide...", MESSAGE_INFORMATION);
        log(mfue);
        return false;
  	} catch(FileNotFoundException exc) {
			message("\nLe fichier des  niveaux n'est pas présent ou pas lisible...\n",MESSAGE_ERREUR);
			log(exc);
			return false; 
		} catch(net.n3.nanoxml.XMLException xe) {
			message("\nLe fichier des niveaux n'est pas valide...\n",MESSAGE_ERREUR);
			log(xe);
			return false;
		} catch(IOException ioe) {
			message("\nErreur de lecture du fichier des niveaux...\n",MESSAGE_ERREUR);
			log(ioe);
			return false;
		} catch(Exception e) {
			message("\nProblème à l'utilisation de la classe de parsage XML...\n",MESSAGE_ERREUR);
			log(e);
			return false;
		}
		return true;		
  }
  
  public Niveau getNiveau(String id) {
    if (id == null || id.equals(""))
      return null;
    return getNiveau(new Integer(id));
  }
  
  public Niveau getNiveau(int id) {
    return getNiveau(new Integer(id));
  }
  
  public Niveau getNiveau(Integer id) {
    return _lesNiveaux.get(id);
  }
  
  public HashMap<Integer, Niveau> getLesNiveaux() {
    return _lesNiveaux;
  }
  
  static public Dimension getDimensionPhoto() {
    return _dimensionPhoto;
  }
  
  public String getConfiguration(String param) {
    String res = _configuration.getConfiguration(param);
    if (res == null)
      return ""; 
		return res;
	}
	
  public boolean setConfiguration(String param, String valeur) {
    return setConfiguration("1", param, valeur);
	}

  public boolean setConfiguration(String rang, String param, String valeur) {
    return _configuration.setConfiguration(rang, param, valeur);
	}
	
	public Classe setClasse(Classe classe) {
    _classe = classe;
    return _classe;
  }
  
  public Classe getClasse() {
    return _classe;
  }
  
  public Classe getClasseById(String id) {
    for(int i = 0; i < _lesClasses.getSize(); i++) 
      if (((Classe)_lesClasses.getElementAt(i)).getId().equals(id))
        return (Classe)_lesClasses.getElementAt(i);
    return null;  
  }
  
	public Eleve setEleve(Eleve eleve) {
    _eleve = eleve;
    return _eleve;
  }
  
  public Eleve getEleve() {
    return _eleve;
  }

	public Epreuve setEpreuve(Epreuve epreuve) {
    _epreuve = epreuve;
    return _epreuve;
  }
  
  public Epreuve getEpreuve() {
    return _epreuve;
  }

	public Evaluation setEvaluation(Evaluation evaluation) {
	 _evaluationCourante = evaluation;
    return _evaluationCourante;
  }
  
  public Evaluation getEvaluation() {
    return _evaluationCourante;
  }
  
  public Vector<Evaluation> getLesEvaluations() {
    return _lesEvaluations;
  }
  
  public JPanel getPFrame() {
    return _pFrame;
  }
  
  public void setTailleCurseur(int i) {
    _tailleCurseur = i;
  }

  public int getTailleCurseur() {
    return _tailleCurseur;
  }

  public void setAvecTbi(boolean oui) {
    _avecTbi = oui;
  }

  public boolean getAvecTbi() {
    return _avecTbi;
  }
  
  public void setLargeurTbi(int v) {
    _largeurTbi = v;
  }

  public int getLargeurTbi() {
    if (!_avecTbi)
      return getBoundsEcran().width;
    return _largeurTbi;
  }

  public void setHauteurTbi(int v) {
    _hauteurTbi = v;
  }

  public int getHauteurTbi() {
    if (!_avecTbi)
      return getBoundsEcran().height;
    return _hauteurTbi;
  }
  
  public void setXTbi(int v) {
    _xTbi = v;
  }

  public int getXTbi() {
    if (!_avecTbi)
      return getBoundsEcran().x;
    return _xTbi;
  }
  
  public void setYTbi(int v) {
    _yTbi = v;
  }

  public int getYTbi() {
    if (!_avecTbi)
      return getBoundsEcran().y;
    return _yTbi;
  }

  public void setNbExercices(int v) {
    _nbExercices = v;
  }

  public int getNbExercices() {
    return _nbExercices;
  }
  
  public int getLargeurLogo() {
    return _largeurLogo;
  }

  public int getHauteurLogo() {
    return _hauteurLogo;
  }
  
  public Dimension getDimensionLogo() {
    return _dimensionLogo;
  }
  
	public PanelJeu getTableauJeu() {
    return _pTableauJeu;
  }
  
  public void addPanneauApresSelectClasse(String panneau) {
    _panneauxApresSelectClasse.add(panneau);
  }
  
  public void clearPanneauApresSelectClasse() {
    _panneauxApresSelectClasse.clear();
  }

  public String getPanneauApresSelectClasse(String pos) {
    if (pos.equals("dernier"))
      return _panneauxApresSelectClasse.elementAt(_panneauxApresSelectClasse.size()-1);
    // renvoit le premier  
    return _panneauxApresSelectClasse.elementAt(0);
  }

  public void addPanneauApresSelectEleve(String panneau) {
    _panneauxApresSelectEleve.add(panneau);
  }
  
  public void clearPanneauApresSelectEleve() {
    _panneauxApresSelectEleve.clear();
  }  

  public String getPanneauApresSelectEleve(String pos) {
    if (pos.equals("dernier"))
      return _panneauxApresSelectEleve.elementAt(_panneauxApresSelectEleve.size()-1);
    // renvoit le premier  
    return _panneauxApresSelectEleve.elementAt(0);
  }

	public void message(String m, int type) {
    if (_init) {
      _message.setMessage(m, type);
      if (type == MESSAGE_AVERTISSEMENT ||
          type == MESSAGE_ERREUR)
          log(m);
    } else {
      if (type == MESSAGE_AVERTISSEMENT ||
          type == MESSAGE_ERREUR)
          log(m);
      messageStatique(m, type);
    }
  }
  
	public void message(Exception e, int type) {
    if (_init) {
      _message.setMessage(e.toString(), type);
      if (type == MESSAGE_AVERTISSEMENT ||
          type == MESSAGE_ERREUR)
          log(e);
    } else {
      if (type == MESSAGE_AVERTISSEMENT ||
          type == MESSAGE_ERREUR)
          log(e);
      messageStatique(e.toString(), type);
    }
  }
  
	public void message(String m, int type, int duree) {
    if (_init) {
      _message.setMessage(m, type, duree);
      if (type == MESSAGE_AVERTISSEMENT ||
          type == MESSAGE_ERREUR)
          log(m);
    } else {
      if (type == MESSAGE_AVERTISSEMENT ||
          type == MESSAGE_ERREUR)
          log(m);
      messageStatique(m, type);
    }
  }
  
	public static void messageStatique(String m, int t) {
      if (t == MESSAGE_ERREUR) 
        JOptionPane.showMessageDialog(null, m, "Message", JOptionPane.ERROR_MESSAGE);
      if (t == MESSAGE_AVERTISSEMENT) 
        JOptionPane.showMessageDialog(null, m, "Message", JOptionPane.WARNING_MESSAGE);
      if (t == MESSAGE_INFORMATION) 
        JOptionPane.showMessageDialog(null, m, "Message", JOptionPane.INFORMATION_MESSAGE);
  }
  
  public URL getRessource(String ress) {
    File res = new File(ress);
    if (res.exists()) {
      try {
        return res.toURI().toURL();
      } catch(MalformedURLException mfue) {
        message(mfue.toString(), MESSAGE_ERREUR);
        log(mfue);
      }      
    }
   URL resURL = getClass().getResource(ress);
   if (resURL == null)
      message("La ressource '" + ress + "' n'a pas été trouvée...", MESSAGE_ERREUR);
	 return resURL; 
  }
  
  public URL getRessourceByConf(String ress) {
	 return getRessource(getConfiguration(ress));
  }

  public void setEcran(GraphicsDevice e, boolean ecranPrincipal) {
    _ecran = e;
    _ecranPrincipal = ecranPrincipal;
  }
  
  public GraphicsDevice getEcran() {
    return _ecran;
  }  

  public boolean isEcranPrincipal() {
    return _ecranPrincipal;
  }  

	public void setPanneau(String p) {
		CardLayout c =(CardLayout)_pMain.getLayout();
		c.show(_pMain, p);
		_pMain.revalidate();
	}
	
	public static TbiApp getThis() { // utilisée par la fonction chargeLesClasses
    return _this;
  }
  
  public void chargeLesClasses(final Panneau pSelectClasse) {
    message("Chargement des classes...", MESSAGE_INFORMATION, 0);
    pSelectClasse.setEnabled(false);
    if (_lesClasses == null)
      _lesClasses = new DefaultListModel();
    else
      _lesClasses.clear();
    (new Thread(new Runnable() {
      public void run() {
        File[] fichiers = null;
        File rep = new File(getConfiguration("chemindesclasses"));
        fichiers = rep.listFiles();
        if (fichiers != null) {
          for(int f = 0; f < fichiers.length; f++) {
            if (fichiers[f].isFile() && fichiers[f].getName().toLowerCase().endsWith("-classe.zip")) {
              Classe classe = new Classe(getThis(), fichiers[f]);
              _lesClasses.addElement(classe);
              ItemClasseGen item = classe.getItemClasse();
              item.setEnabled(false);
            }
          }
        }
        SwingUtilities.invokeLater(new Runnable() {
          public void run() {
            pSelectClasse.setEnabled(true);
            if (_lesClasses.getSize() > 0)
              message("Chargement des classes : " + _lesClasses.getSize() + " trouvée" + (_lesClasses.getSize() > 1 ? "s" : "") + " !", MESSAGE_INFORMATION, 3000);
            else 
              message("Chargement des classes : aucune classe trouvée...", MESSAGE_AVERTISSEMENT, 3000);
            chargeLesEvaluations();              
          }
        });
      };
    })).start();
  }

  public boolean getUtilisationDesClasses() {
    return _utilisationDesClasses;
  }
  
  public void setUtilisationDesClasses(boolean u) {
    _utilisationDesClasses = u;
  }
  
  public String getCheminDesClasses() {
    return _cheminDesClasses;
  }

  public void setCheminDesClasses(String chemin) {
    _cheminDesClasses = chemin;
  }

  public String getCheminDesEvaluations() {
    return _cheminDesClasses + File.separator + "evaluations" + File.separator + _applicationInfo.getNom();
  }
  
  public DefaultListModel getLesClasses() {
    return _lesClasses;
  }
   
// Partie Audio  
  
  public void play(URL url) {
    _oggPlayer.play(url);
  } 

  public void stopPlayer() {
    _oggPlayer.stop(); 
  }
  
  public boolean isPlaying() {
    return !_oggPlayer.pret();
  }

  public boolean sauveConfiguration() {
    /*************************************
     * 0 : TbiGeneralApp.conf    
     * 1 : TbiGeneral.conf
     * 2 : Tbi.......App.conf     
     * 3 : Tbi........conf
     *************************************/
    setConfiguration("1", _V_TailleSouris, Integer.toString(_tailleCurseur));
    setConfiguration("1", _V_UtilisationTBI, _avecTbi ? "oui" : "non");
    setConfiguration("1", _V_LargeurTBI, Integer.toString(_largeurTbi));    
    setConfiguration("1", _V_HauteurTBI, Integer.toString(_hauteurTbi));
    setConfiguration("1", _V_CoordXTBI, Integer.toString(_xTbi));    
    setConfiguration("1", _V_CoordYTBI, Integer.toString(_yTbi));
    setConfiguration("1", _V_CheminDesClasses, _cheminDesClasses);
    setConfiguration("1", _V_NbExercices, Integer.toString(_nbExercices));
    return sauveConfiguration("1");
  }
  
  protected boolean sauveConfiguration(String rang) {
    /*************************************
     * 0 : TbiGeneralApp.conf    
     * 1 : TbiGeneral.conf
     * 2 : Tbi.......App.conf     
     * 3 : Tbi........conf
     *************************************/
    boolean res = _configuration.sauveConfiguration(rang);
    if (res)
      message("Enregistrement de la configuration bien effectué", MESSAGE_INFORMATION);
    else 
      message("Problème lors de l'enregistrement de la configuration", MESSAGE_ERREUR); 
    return res;
  }
  
  public void initSuivantConfiguration() {
    if (getConfiguration(_V_TailleSouris).equals("1"))
      setTailleCurseur(1);
    else if (getConfiguration(_V_TailleSouris).equals("2")) 
      setTailleCurseur(2);
    else 
      setTailleCurseur(3);
    if (getConfiguration(_V_UtilisationTBI).toLowerCase().equals("oui"))
      setAvecTbi(true);
    else 
      setAvecTbi(false);
    if (getConfiguration(_V_LargeurTBI) != null)
      setLargeurTbi(Integer.parseInt(getConfiguration(_V_LargeurTBI)));
    if (getConfiguration(_V_HauteurTBI) != null)
      setHauteurTbi(Integer.parseInt(getConfiguration(_V_HauteurTBI)));
    if (getConfiguration(_V_CoordXTBI) != null)
      setXTbi(Integer.parseInt(getConfiguration(_V_CoordXTBI)));
    if (getConfiguration(_V_CoordYTBI) != null)
      setYTbi(Integer.parseInt(getConfiguration(_V_CoordYTBI)));
    if (getConfiguration(_V_UtilisationDesClasses).toLowerCase().equals("non"))
      setUtilisationDesClasses(false);
    else 
      setUtilisationDesClasses(true);
    if (getConfiguration(_V_CheminDesClasses) != null)
      setCheminDesClasses(getConfiguration(_V_CheminDesClasses));
    chargeLesSkins();  
    if (!getConfiguration(_V_NomDuSkin).equals(""))
      setSkin(getConfiguration(_V_NomDuSkin));
    else 
      setSkin(_nomDuSkin);  
  }
  
  public void selectionneEleve(Eleve e, boolean oui) {
    if (oui && !e.getItemEleve().estSelectionne()) {
      setEleve(e);
      if (_evaluationCourante != null) {
        _epreuve = _evaluationCourante.addEpreuve(getNewEpreuve());
        _epreuve.start();
      }
      setPanneau(getPanneauApresSelectEleve("premier"));
    }
  }  
  
  public Epreuve getNewEpreuve() {
    return new Epreuve(this);
  }

  public Epreuve getNewEpreuve(Classe classe, IXMLElement xml) throws Exception {
    return new Epreuve(this, classe, xml);
  }

  public Evaluation getNewEvaluation() {
    return new Evaluation(this);
  }

  public Evaluation getNewEvaluation(IXMLElement xml) throws Exception {
    Evaluation eval = new Evaluation(this);
    eval.chargeEvaluation(xml);
    return eval;
  }
  
  public Evaluation getNewEvaluation(String fichier) throws Exception {
    return new Evaluation(this, fichier);
  }
  
  public boolean demarreNouvelleEvaluation() {
    if (getUtilisationDesClasses()) {
      int rep = JOptionPane.showConfirmDialog(_pFrame, "Souhaitez-vous démarrer une évaluation ?", "Nouvelle évaluation ?", JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE);
      if (_evaluationCourante != null) {
        if (_evaluationCourante.peutSeTerminer()) {
          _lesEvaluations.add(_evaluationCourante);
          try {
            _evaluationCourante.enregistrer(getCheminDesEvaluations());
            message("Enregistrement de l'évaluation " + _evaluationCourante.getNom() + " réussi...", MESSAGE_INFORMATION);
          } catch (Exception e) {
            message("Problème lors de l'enregistrement de l'évaluation " + _evaluationCourante.getNom() + "... " + e.toString(), MESSAGE_ERREUR);
          }
        } else {
          message("Une évaluation est en cours...", MESSAGE_AVERTISSEMENT);
          return false;
        }
      }
      if (rep == JOptionPane.NO_OPTION) 
        return false;
      Evaluation ev = setEvaluation(getNewEvaluation());
      initEvaluation(ev);
      ev.start();
      return true;
    }
    return false;
  }
  
  public void chargeLesEvaluations() {
    message("Chargement des évaluations...", MESSAGE_INFORMATION, 0);
    _lesEvaluations.clear();
    File[] fichiers = null;
    File rep = new File(getCheminDesEvaluations());
    fichiers = rep.listFiles();
    int cptErreur = 0;
    if (fichiers != null) {
      for(int f = 0; f < fichiers.length; f++) {
        if (fichiers[f].isFile() && fichiers[f].getName().endsWith("-eval.xml")) {
          try {
            Evaluation eval = getNewEvaluation(fichiers[f].getAbsolutePath());
           _lesEvaluations.addElement(eval);
          } catch(Exception e) {
            message(e.toString(), MESSAGE_AVERTISSEMENT, 3000);
            cptErreur++;
          }
        }
      }
    }
    if (_lesEvaluations.size() > 0) {
      // tri des évaluations de la plus récente à la plus ancienne
      ArrayList<ParametreDeTri> lesParametres = Evaluation.getParametresDeTri();
      Iterator it = lesParametres.iterator();
      while (it.hasNext()) {
        ParametreDeTri p = (ParametreDeTri)it.next();
        if (!p.getIntitule().equals("Date récente->ancienne"))
          lesParametres.remove(lesParametres.indexOf(p));
      }
      Comparator<Evaluation> cp = Evaluation.getComparator(lesParametres);
      Collections.sort(_lesEvaluations, cp);
      message("Chargement des évaluations : " + _lesEvaluations.size() + " trouvée" + (_lesEvaluations.size() > 1 ? "s" : "") + " !" + (cptErreur > 0 ? " (+ " + cptErreur + " erronnée" + (cptErreur > 1 ? "s" : "") + ")" : ""), (cptErreur > 0 ? MESSAGE_AVERTISSEMENT : MESSAGE_INFORMATION), 3000);
    } else 
      message("Chargement des évaluations : aucune évaluation trouvée..." + (cptErreur > 0 ? " (+ " + cptErreur + " erronnée" + (cptErreur > 1 ? "s" : "") + ")" : ""), (cptErreur > 0 ? MESSAGE_AVERTISSEMENT : MESSAGE_INFORMATION), 3000);
  }
  
  public boolean enregistrerLesEvaluations() {
    for(int i = 0; i < _lesEvaluations.size(); i++) {
      Evaluation ev = _lesEvaluations.elementAt(i);
      if (!ev.estEnregistree() && ev.peutSeTerminer()) {
        try {
          ev.enregistrer(getCheminDesEvaluations());
        } catch(Exception e) {
          e.printStackTrace();
          message("Erreur lors de l'enregistrement de l'évaluation : " + e.toString(), MESSAGE_ERREUR);
          return false; 
        }
      }
    }
    if (!_evaluationCourante.estEnregistree() && _evaluationCourante.peutSeTerminer()) {
      try {
        _evaluationCourante.enregistrer(getCheminDesEvaluations());
      } catch(Exception e) {
        e.printStackTrace();
        message("Erreur lors de l'enregistrement de l'évaluation : " + e.toString(), MESSAGE_ERREUR);
        return false; 
      }
    }
    return true;
  }
  
  public void annuleEpreuve() {
    if (_epreuve != null && _evaluationCourante != null) {
      _evaluationCourante.annuleEpreuve(_epreuve);
    }
    setEpreuve(null);
  }
  
  public void initEvaluation(Evaluation ev) {
    ev.setNom(genereNomEvaluation());
    ev.setCle(genereCleEvaluation());
    ev.setVersionApplication(genereVersionEvaluation());
    if (getSkin() != null)
      ev.setLangue(getSkin().toString());
  }

  public String genereCleEvaluation() {
    return getApplicationInfo().getNom() + "-";
  }

  public String genereNomEvaluation() {
    return "Evaluation";
  }
  
  public String genereVersionEvaluation() {
    return getApplicationInfo().getVersion();
  }
  
  public ItemMire genereItemMire(PanelJeu pJeu) {
    return new ItemMire(pJeu);
  }
  
  public void genereMiresEvaluations(PanelJeu pJeu, ItemMire mireCourante, Box pMires) {
    pMires.removeAll();
    mireCourante.setAlignmentX(Component.LEFT_ALIGNMENT);
    pMires.add(mireCourante);
    for(int i = 0; i < _lesEvaluations.size(); i++) {
      Evaluation ev = _lesEvaluations.elementAt(i);
      if (ev != getEvaluation()) {
        if (ev.getCle().equals(genereCleEvaluation())) {
          Vector<Epreuve> lesEpreuves = ev.getLesEpreuves();
          for(int j = 0; j < lesEpreuves.size(); j++) {
            Epreuve ep = lesEpreuves.elementAt(j);
            if (ep.getEleve().getId().equals(getEleve().getId())) {
              ItemMire iMire = genereItemMire(pJeu);
              iMire.setNbExercicesCorrects(ep.getNbSucces());
              iMire.setNbExercices(ev.getNbExercices());
              iMire.setDate(ep.getDateStart());
              iMire.setAlignmentX(Component.LEFT_ALIGNMENT);
              pMires.add(iMire);
            }
          }
        }
      }
    }    
  }
  
  public ItemEleveGen genereItemEleve(Eleve e) {
    ItemEleve item = new ItemEleve(e);
    item.addActionListener(this);
    return item;
  }

  public ItemClasseGen genereItemClasse(Classe c) {
    ItemClasse item = new ItemClasse(c);
    return item;
  }
  
  public boolean verifieUniciteIdClasse(Classe classe, String id) {
    for(int i = 0; i < _lesClasses.size(); i++) {
      Classe c = (Classe)_lesClasses.get(i);
      if (c.getId().equals(id) && classe != c)
        return false;
    }
    return true;
  }
  
  public Vector<Eleve> getElevesSelectionnes() { // utilisée par TbiAdmin
    return null; 
  }
    
  public Classe getClasseSelectionnee() { // utilisée par TbiAdmin
    return null; 
  }
  
  static public String getJourDeLaSemaine(Calendar date) {
    switch (date.get(Calendar.DAY_OF_WEEK)) {
      case Calendar.MONDAY : return "Lundi";
      case Calendar.TUESDAY : return "Mardi";
      case Calendar.WEDNESDAY : return "Mercredi";
      case Calendar.THURSDAY : return "Jeudi";
      case Calendar.FRIDAY : return "Vendredi";
      case Calendar.SATURDAY : return "Samedi";
      case Calendar.SUNDAY : return "Dimanche";
    }
    return "";
  }

  
}