/*
Copyright Caroline et Serge COUDÉ, (03/2012) 

serge.coude@gmail.com caroline.coude@laposte.net

Ce logiciel est un programme informatique faisant partie d'une suite de 
logiciels 'TbiSuite' à destination des enseignants et des élèves usagés des 
ordinateurs et Tableau Numérique Interactif / Tableau Blanc Interactif. 

Ce logiciel est régi par la licence CeCILL v2 soumise au droit français et
respectant les principes de diffusion des logiciels libres. Vous pouvez
utiliser, modifier et/ou redistribuer ce programme sous les conditions
de la licence CeCILL v2 telle que diffusée par le CEA, le CNRS et l'INRIA 
sur le site "http://www.cecill.info".

En contrepartie de l'accessibilité au code source et des droits de copie,
de modification et de redistribution accordés par cette licence, il n'est
offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
seule une responsabilité restreinte pèse sur l'auteur du programme,  le
titulaire des droits patrimoniaux et les concédants successifs.

A cet égard  l'attention de l'utilisateur est attirée sur les risques
associés au chargement,  à l'utilisation,  à la modification et/ou au
développement et à la reproduction du logiciel par l'utilisateur étant 
donné sa spécificité de logiciel libre, qui peut le rendre complexe à 
manipuler et qui le réserve donc à des développeurs et des professionnels
avertis possédant  des  connaissances  informatiques approfondies.  Les
utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
logiciel à leurs besoins dans des conditions permettant d'assurer la
sécurité de leurs systèmes et ou de leurs données et, plus généralement, 
à l'utiliser et l'exploiter dans les mêmes conditions de sécurité. 

Le fait que vous puissiez accéder à cet en-tête signifie que vous avez 
pris connaissance de la licence CeCILL v2, et que vous en avez accepté les
termes.

*/

package net.sergecoude.TbiQuizzSonore;

import java.util.*;
import java.io.*;
import net.n3.nanoxml.*;
import net.sergecoude.TbiGeneral.*;

public class EvaluationQuizz extends Evaluation  {

  private final static long serialVersionUID = 1L;
  
  private int _nbImages;
  private boolean _nomPrononce;

  public EvaluationQuizz(TbiQuizzSonore app) {
    super(app);
  }
  
  public EvaluationQuizz(TbiQuizzSonore app, String chemin) throws Exception {
    super(app, chemin);      
  }

  public EvaluationQuizz(TbiApp app) {
    super(app);
  }
  
  public EvaluationQuizz(TbiApp app, String chemin) throws Exception {
    super(app, chemin);      
  }
  
  public void chargeEvaluation(IXMLElement xml) throws Exception {
    super.chargeEvaluation(xml);
		IXMLElement xNbImages = xml.getFirstChildNamed("nbimages");
		if (xNbImages != null) {
		  setNbImages(Integer.parseInt(xNbImages.getContent()));
    } else 
      throwNewException("Le nombre d'images de l'évaluation n'est pas indiqué... [" + _chemin + "]");
		IXMLElement xNomPrononce = xml.getFirstChildNamed("nomprononce");
		if (xNomPrononce != null) {
		  setNomPrononce(Integer.parseInt(xNomPrononce.getContent()) == 1 ? true : false);
    } else 
      throwNewException("Le nom prononcé ou pas de l'évaluation n'est pas indiqué... [" + _chemin + "]");
  }
  
  @Override
  public Epreuve getNewEpreuve(Classe classe, IXMLElement xmlEpreuve) throws Exception {
    return new EpreuveQuizz(_app, classe, xmlEpreuve);
  }  
  
  public void setNbImages(int nbImages) {
    _nbImages = nbImages;
  }
  
  public int getNbImages() {
    return _nbImages;
  }
  
  public void setNomPrononce(boolean nomPrononce) {
    _nomPrononce = nomPrononce;
  }
  
  public boolean getNomPrononce() {
    return _nomPrononce;
  }
  
  public String toXmlDebut() {
    String xml = "<nbimages>" + getNbImages() + "</nbimages>";
    xml += "<nomprononce>" + (getNomPrononce() ? "1" : "0") + "</nomprononce>";
    return super.toXmlDebut() + xml;
  }

  @Override
  public String getCodeNiveauDifficulte() {
    StringTokenizer st = new StringTokenizer(getCle(), "-");
    if (st.hasMoreTokens()) {
      st.nextToken(); // le nom de l'appli
      if (st.hasMoreTokens()) {
        st.nextToken(); // le thème
        String code = ""; 
        if (st.hasMoreTokens()) {
          String c1 = st.nextToken();
          String c2 = "";
          while(st.hasMoreTokens()) {
            c2 = st.nextToken();
            if (st.hasMoreTokens()) 
              c1 = c2;
          }           
          code += c1 + "-" + c2;
          return code;
        } 
      }
    }
    return "";
  }
   
}
